@@ -1,8 +1,10 @@
 """Transformer computing second-order attributes."""
+from __future__ import annotations
 
 import logging
 import math
 from collections import defaultdict
+from typing import Any, Dict, Iterable, List
 
 import numpy as np
 from sklearn.base import BaseEstimator, TransformerMixin
@@ -13,15 +15,17 @@ logger = logging.getLogger(__name__)
 class SubFrequencyVectorizer(BaseEstimator, TransformerMixin):
     """Transformer computing second-order attributes."""
 
-    def __init__(self):
+    def __init__(self) -> None:
         """Initialize the model."""
-        self.t = defaultdict(np.array)
+        self.t: Dict[str, np.array] = defaultdict(np.array)
 
-    def fit(self, X, y=None, **fit_params):
+    def fit(
+        self, X: List[str], y: Iterable[str], **_fit_params: Any
+    ) -> SubFrequencyVectorizer:
         """Fit data to model."""
         words = set()
 
-        logger.info('starting phase 0')
+        logger.info("starting phase 0")
         documents = defaultdict(list)
         for document, target in zip(X, y):
             documents[target].append(document)
@@ -29,8 +33,8 @@ class SubFrequencyVectorizer(BaseEstimator, TransformerMixin):
                 words.add(word)
 
         targets = set(y)
-        tp = defaultdict(dict)
-        weights = defaultdict(lambda: defaultdict(float))
+        tp: Dict[str, Dict[str, float]] = defaultdict(lambda: defaultdict(float))
+        weights: Dict[str, Dict[str, float]] = defaultdict(lambda: defaultdict(float))
 
         # equation 1: weight calculation
         for target, target_docs in documents.items():
@@ -44,7 +48,7 @@ class SubFrequencyVectorizer(BaseEstimator, TransformerMixin):
         # equation 2.2: normalization
         for target in targets:
             for word in words:
-                norm_term = 0
+                norm_term = 0.0
                 for k in targets:
                     norm_term += weights[k][word]
                 tp[target][word] = weights[target][word] / norm_term
@@ -54,7 +58,7 @@ class SubFrequencyVectorizer(BaseEstimator, TransformerMixin):
 
         return self
 
-    def transform(self, X):
+    def transform(self, X: Iterable[str], _y: Any = None) -> np.array:
         """Transform data due to previously learned frequencies."""
         result = []
         for k in X:
